from graphics import ChessSquare, ChessSound, ChessMove
from piece import *

import copy
import os

class chessBoard:
    def __init__(self):
        self.boardSpaces = [[0, 0, 0, 0, 0, 0, 0, 0] for col in range(8)]
        self.previousMove = None
        self.createBoard()
        self.addPieces('white')
        self.addPieces('black')

def handle_client(conn, addr):
    board = chess.Board()
    print(f"Client {addr} connected")
    conn.sendall(str(board).encode())

    while not board.is_game_over():
        if board.turn == chess.WHITE:
            data = conn.recv(1024).decode()
            move = chess.Move.from_uci(data)
            board.push(move)
            conn.sendall(str(board).encode())
        else:
            print("AI is thinking...")
            move = ai_move(board, depth=3)
            board.push(move)
            conn.sendall(str(board).encode())

    print(f"Client {addr} disconnected")
    conn.close()

def start_server():
    server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server.bind(('localhost', 5000))
    server.listen(1)
    print("Server started")

    while True:
        conn, addr = server.accept()
        handle_client(conn, addr)

start_server()



    def makeMove(self, piece, moveObj, isTest=False):
        startSquare = moveObj.start
        endSquare = moveObj.end

        enPassantEmpty = self.boardSpaces[endSquare.row][endSquare.col].isEmpty()

        # update board move on console
        self.boardSpaces[startSquare.row][startSquare.col].piece = None
        self.boardSpaces[endSquare.row][endSquare.col].piece = piece

        if isinstance(piece, Pawn):
            # en passant capture
            columnDiff = endSquare.col - startSquare.col
            if columnDiff != 0 and enPassantEmpty:
                # update board move on console
                self.boardSpaces[startSquare.row][startSquare.col + columnDiff].piece = None
                self.boardSpaces[endSquare.row][endSquare.col].piece = piece
                if not isTest:
                    captureSound = ChessSound(
                        os.path.join('assets/sounds/capture.wav'))
                    captureSound.play()

            # pawn promotion
            else:
                self.checkPromotion(piece, endSquare)

        # king castling
        if isinstance(piece, King):
            if self.performCastling(startSquare, endSquare) and not isTest:
                diff = endSquare.col - startSquare.col
                currentRook = piece.leftRook if (diff < 0) else piece.rightRook
                self.makeMove(currentRook, currentRook.moveList[-1])

        # move
        piece.hasMoved = True

        # clear valid moves
        piece.clearMoves()

        # set last move
        self.previousMove = moveObj

    def isValidMove(self, piece, moveObj):
        return moveObj in piece.moveList
    
    def evaluate(self):
        evaluation = 0
        for row in range(8):
            for col in range(8):
                square = self.boardSpaces[row][col]
                if square.hasPiece():
                    piece = square.piece
                    piece_value = piece.value
                    if piece.color == 'white':
                        evaluation += piece_value
                    else:
                        evaluation -= piece_value
        print(f"Board evaluation: {evaluation}")
        return evaluation

    
    def getValidMoves(self, row, col):
        piece = self.boardSpaces[row][col].piece
        if piece is None:
            return []

        moves = piece.getValidMoves(row, col, self)
        print(f"Piece: {piece}, Moves: {moves}")
        return moves


    def generateMoves(self, player):
        print(f"Player: {player}")
        # ...
        moves = []
        for row in range(8):
            for col in range(8):
                square = self.boardSpaces[row][col]
                if square.hasPiece() and square.piece.color == player:
                    piece_moves = self.getValidMoves(row, col)
                    moves.extend(piece_moves)
                    print(f"Piece: {square.piece}, Moves: {piece_moves}")

        print(f"Generated moves for {player}: {moves}")
        return moves




    def checkPromotion(self, piece, endSquare):
        if endSquare.row == 0 or endSquare.row == 7:
            self.boardSpaces[endSquare.row][endSquare.col].piece = Queen(piece.color)

    def performCastling(self, startSquare, endSquare):
        return abs(startSquare.col - endSquare.col) == 2

    def setTrueEnPassant(self, piece):

        if not isinstance(piece, Pawn):
            return

        for row in range(8):
            for col in range(8):
                if isinstance(self.boardSpaces[row][col].piece, Pawn):
                    self.boardSpaces[row][col].piece.enPassant = False

        piece.enPassant = True

    def isInCheck(self, piece, moveObj):
        tempPiece = copy.deepcopy(piece)
        tempBoard = copy.deepcopy(self)
        tempBoard.makeMove(tempPiece, moveObj, isTest=True)

        for row in range(8):
            for col in range(8):
                if tempBoard.boardSpaces[row][col].hasOpponentPiece(piece.color):
                    opponentPiece = tempBoard.boardSpaces[row][col].piece
                    tempBoard.computeMoves(opponentPiece, row, col, checkForCheck=False)
                    for move in opponentPiece.moveList:
                        if isinstance(move.end.piece, King):
                            return True

        return False


    def computeMoves(self, piece, rowNum, colNum, checkForCheck=True):
        def pawnMoves():
            # steps
            stepCount = 1 if piece.hasMoved else 2

            # vertical moves
            startPosition = rowNum + piece.direction
            endPosition = rowNum + (piece.direction * (1 + stepCount))
            for possibleMoveRow in range(startPosition, endPosition, piece.direction):
                if ChessSquare.isInRange(possibleMoveRow):
                    if self.boardSpaces[possibleMoveRow][colNum].isEmpty():
                        # create initial and final move squares
                        startSquare = ChessSquare(rowNum, colNum)
                        endSquare = ChessSquare(possibleMoveRow, colNum)
                        # create a new move
                        newMove = ChessMove(startSquare, endSquare)

                        # check potential checks
                        if checkForCheck:
                            if not self.isInCheck(piece, newMove):
                                # append new move
                                piece.addMove(newMove)
                        else:
                            # append new move
                            piece.addMove(newMove)
                    # blocked
                    else:
                        break
                # not in range
                else:
                    break

            # diagonal moves
            possibleMoveRow = rowNum + piece.direction
            possibleMoveCols = [colNum-1, colNum+1]
            for possibleMoveCol in possibleMoveCols:
                if ChessSquare.isInRange(possibleMoveRow, possibleMoveCol):
                    if self.boardSpaces[possibleMoveRow][possibleMoveCol].hasOpponentPiece(piece.color):
                        # create initial and final move squares
                        startSquare = ChessSquare(rowNum, colNum)
                        finalPiece = self.boardSpaces[possibleMoveRow][possibleMoveCol].piece
                        endSquare = ChessSquare(possibleMoveRow, possibleMoveCol, finalPiece)
                        # create a new move
                        newMove = ChessMove(startSquare, endSquare)
                        
                        # check potential checks
                        if checkForCheck:
                            if not self.isInCheck(piece, newMove):
                                # append new move
                                piece.addMove(newMove)
                        else:
                            # append new move
                            piece.addMove(newMove)

            # en passant moves
            enPassantRow = 3 if piece.color == 'white' else 4
            finalRow = 2 if piece.color == 'white' else 5
            # left en passant
            if ChessSquare.isInRange(colNum-1) and rowNum == enPassantRow:
                if self.boardSpaces[rowNum][colNum-1].hasOpponentPiece(piece.color):
                    capturedPiece = self.boardSpaces[rowNum][colNum-1].piece
                    if isinstance(capturedPiece, Pawn):
                        if capturedPiece.enPassant:
                            # create initial and final move squares
                            startSquare = ChessSquare(rowNum, colNum)
                            endSquare = ChessSquare(finalRow, colNum-1, capturedPiece)
                            # create a new move
                            newMove = ChessMove(startSquare, endSquare)
                            
                            # check potential checks
                            if checkForCheck:
                                if not self.isInCheck(piece, newMove):
                                    # append new move
                                    piece.addMove(newMove)
                            else:
                                # append new move
                                piece.addMove(newMove)

            # right en passant
            if ChessSquare.isInRange(colNum+1) and rowNum == enPassantRow:
                if self.boardSpaces[rowNum][colNum+1].hasOpponentPiece(piece.color):
                    capturedPiece = self.boardSpaces[rowNum][colNum+1].piece
                    if isinstance(capturedPiece, Pawn):
                        if capturedPiece.enPassant:
                        # create initial and final move squares
                            startSquare = ChessSquare(rowNum, colNum)
                            endSquare = ChessSquare(finalRow, colNum+1, capturedPiece)
                            # create a new move
                            newMove = ChessMove(startSquare, endSquare)
                            # check potential checks
                            if checkForCheck:
                                if not self.isInCheck(piece, newMove):
                                    # append new move
                                    piece.addMove(newMove)
                            else:
                                # append new move
                                piece.addMove(newMove)    

        def knightMoves():
            # 8 possible moves
            possibleMoves = [
                (rowNum-2, colNum+1),
                (rowNum-1, colNum+2),
                (rowNum+1, colNum+2),
                (rowNum+2, colNum+1),
                (rowNum+2, colNum-1),
                (rowNum+1, colNum-2),
                (rowNum-1, colNum-2),
                (rowNum-2, colNum-1),
            ]

            for potentialMove in possibleMoves:
                possibleMoveRow, possibleMoveCol = potentialMove

                if ChessSquare.isInRange(possibleMoveRow, possibleMoveCol):
                    if self.boardSpaces[possibleMoveRow][possibleMoveCol].isEmptyOrOpponent(piece.color):
                        # create squares of the new move
                        startSquare = ChessSquare(rowNum, colNum)
                        finalPiece = self.boardSpaces[possibleMoveRow][possibleMoveCol].piece
                        endSquare = ChessSquare(possibleMoveRow, possibleMoveCol, finalPiece)
                        # create new move
                        newMove = ChessMove(startSquare, endSquare)
                        
                        # check potential checks
                        if checkForCheck:
                            if not self.isInCheck(piece, newMove):
                                # append new move
                                piece.addMove(newMove)
                            else:
                                break
                        else:
                            # append new move
                            piece.addMove(newMove)

        def linearMoves(incrList):
            for increment in incrList:
                rowIncr, colIncr = increment
                potentialMoveRow = rowNum + rowIncr
                potentialMoveCol = colNum + colIncr

                while True:
                    if ChessSquare.isInRange(potentialMoveRow, potentialMoveCol):
                        # create squares for the potential new move
                        startSquare = ChessSquare(rowNum, colNum)
                        endPiece = self.boardSpaces[potentialMoveRow][potentialMoveCol].piece
                        endSquare = ChessSquare(potentialMoveRow, potentialMoveCol, endPiece)
                        # create a potential new move
                        moveObj = ChessMove(startSquare, endSquare)

                        # empty = continue looping
                        if self.boardSpaces[potentialMoveRow][potentialMoveCol].isEmpty():
                            # check for potential checks
                            if checkForCheck:
                                if not self.isInCheck(piece, moveObj):
                                    # append new move
                                    piece.addMove(moveObj)
                            else:
                                # append new move
                                piece.addMove(moveObj)

                        # has enemy piece = add move + break
                        elif self.boardSpaces[potentialMoveRow][potentialMoveCol].hasOpponentPiece(piece.color):
                            # check for potential checks
                            if checkForCheck:
                                if not self.isInCheck(piece, moveObj):
                                    # append new move
                                    piece.addMove(moveObj)
                            else:
                                # append new move
                                piece.addMove(moveObj)
                            break

                        # has team piece = break
                        elif self.boardSpaces[potentialMoveRow][potentialMoveCol].hasTeamPiece(piece.color):
                            break

                    # not in range
                    else: break

                    # incrementing increments
                    potentialMoveRow = potentialMoveRow + rowIncr
                    potentialMoveCol = potentialMoveCol + colIncr


        def kingMoves():
            adjacentSpaces = [
            (rowNum-1, colNum+0), # up
            (rowNum-1, colNum+1), # up-right
            (rowNum+0, colNum+1), # right
            (rowNum+1, colNum+1), # down-right
            (rowNum+1, colNum+0), # down
            (rowNum+1, colNum-1), # down-left
            (rowNum+0, colNum-1), # left
            (rowNum-1, colNum-1), # up-left
            ]

            # normal moves
            for possibleMove in adjacentSpaces:
                potentialMoveRow, potentialMoveCol = possibleMove

                if ChessSquare.isInRange(potentialMoveRow, potentialMoveCol):
                    if self.boardSpaces[potentialMoveRow][potentialMoveCol].isEmptyOrOpponent(piece.color):
                        # create squares for the new move
                        startSquare = ChessSquare(rowNum, colNum)
                        endSquare = ChessSquare(potentialMoveRow, potentialMoveCol)
                        # create new move
                        moveObj = ChessMove(startSquare, endSquare)
                        # check for potential checks
                        if checkForCheck:
                            if not self.isInCheck(piece, moveObj):
                                # append new move
                                piece.addMove(moveObj)
                            else: break
                        else:
                            # append new move
                            piece.addMove(moveObj)

            # castling moves
            if not piece.hasMoved:
                # queen castling
                leftRook = self.boardSpaces[rowNum][0].piece
                if isinstance(leftRook, Rook):
                    if not leftRook.hasMoved:
                        for c in range(1, 4):
                            # castling is not possible because there are pieces in between ?
                            if self.boardSpaces[rowNum][c].hasPiece():
                                break

                            if c == 3:
                                # adds left rook to king
                                piece.leftRook = leftRook

                                # rook move
                                startSquare = ChessSquare(rowNum, 0)
                                endSquare = ChessSquare(rowNum, 3)
                                moveR = ChessMove(startSquare, endSquare)

                                # king move
                                startSquare = ChessSquare(rowNum, colNum)
                                endSquare = ChessSquare(rowNum, 2)
                                moveK = ChessMove(startSquare, endSquare)

                                # check for potential checks
                                if checkForCheck:
                                    if not self.isInCheck(piece, moveK) and not self.isInCheck(leftRook, moveR):
                                        # append new move to rook
                                        leftRook.addMove(moveR)
                                        # append new move to king
                                        piece.addMove(moveK)
                                else:
                                    # append new move to rook
                                    leftRook.addMove(moveR)
                                    # append new move king
                                    piece.addMove(moveK)

                # king castling
                rightRook = self.boardSpaces[rowNum][7].piece
                if isinstance(rightRook, Rook):
                    if not rightRook.hasMoved:
                        for c in range(5, 7):
                            # castling is not possible because there are pieces in between ?
                            if self.boardSpaces[rowNum][c].hasPiece():
                                break

                            if c == 6:
                                # adds right rook to king
                                piece.rightRook = rightRook

                                # rook move
                                startSquare = ChessSquare(rowNum, 7)
                                endSquare = ChessSquare(rowNum, 5)
                                moveR = ChessMove(startSquare, endSquare)

                                # king move
                                startSquare = ChessSquare(rowNum, colNum)
                                endSquare = ChessSquare(rowNum, 6)
                                moveK = ChessMove(startSquare, endSquare)

                                # check for potential checks
                                if checkForCheck:
                                    if not self.isInCheck(piece, moveK) and not self.isInCheck(rightRook, moveR):
                                        # append new move to rook
                                        rightRook.addMove(moveR)
                                        # append new move to king
                                        piece.addMove(moveK)
                                else:
                                    # append new move to rook
                                    rightRook.addMove(moveR)
                                    # append new move king
                                    piece.addMove(moveK)

        if isinstance(piece, Pawn):
            pawnMoves()

        elif isinstance(piece, Knight):
            knightMoves()

        elif isinstance(piece, Bishop):
            linearMoves([
            (-1, 1), # up-right
            (-1, -1), # up-left
            (1, 1), # down-right
            (1, -1), # down-left
            ])

        elif isinstance(piece, Rook):
            linearMoves([
            (-1, 0), # up
            (0, 1), # right
            (1, 0), # down
            (0, -1), # left
            ])

        elif isinstance(piece, Queen):
            linearMoves([
            (-1, 1), # up-right
            (-1, -1), # up-left
            (1, 1), # down-right
            (1, -1), # down-left
            (-1, 0), # up
            (0, 1), # right
            (1, 0), # down
            (0, -1) # left
            ])

        elif isinstance(piece, King):
            kingMoves()

    def createBoard(self):
        for row in range(8):
            for col in range(8):
                self.boardSpaces[row][col] = ChessSquare(row, col)

    def addPieces(self, color):
        pawnRow, otherRow = (6, 7) if color == 'white' else (1, 0)
        # pawns
        for col in range(8):
            self.boardSpaces[pawnRow][col] = ChessSquare(pawnRow, col, Pawn(color))

        # knights
        self.boardSpaces[otherRow][1] = ChessSquare(otherRow, 1, Knight(color))
        self.boardSpaces[otherRow][6] = ChessSquare(otherRow, 6, Knight(color))

        # bishops
        self.boardSpaces[otherRow][2] = ChessSquare(otherRow, 2, Bishop(color))
        self.boardSpaces[otherRow][5] = ChessSquare(otherRow, 5, Bishop(color))

        # rooks
        self.boardSpaces[otherRow][0] = ChessSquare(otherRow, 0, Rook(color))
        self.boardSpaces[otherRow][7] = ChessSquare(otherRow, 7, Rook(color))

        # queen
        self.boardSpaces[otherRow][3] = ChessSquare(otherRow, 3, Queen(color))

        # king
        self.boardSpaces[otherRow][4] = ChessSquare(otherRow, 4, King(color))


