import pygame

from graphics import ChessDragger, ChessSquare, ChessConfig
from board import chessBoard


screenWidth = 800
screenHeight = 800
boardRows = 8
boardCols = 8
squareSize = screenWidth // boardCols

class ChessGame:
    def __init__(self):
        self.currentPlayer = 'white'
        self.hoveredSquare = None
        self.board = chessBoard()
        self.pieceDragger = ChessDragger()
        self.config = ChessConfig()
        self.single_player = True 

    def drawBackground(self, surface):
        theme = self.config.theme
        
        for row in range(8):
            for col in range(8):
                # color
                color = theme.bg.light if (row + col) % 2 == 0 else theme.bg.dark
                # rect
                rect = (col * squareSize, row * squareSize, squareSize, squareSize)
                # draw
                pygame.draw.rect(surface, color, rect)

                # row coordinates
                if col == 0:
                    # color
                    color = theme.bg.dark if row % 2 == 0 else theme.bg.light
                    # label
                    label = self.config.font.render(str(8-row), 1, color)
                    labelPos = (5, 5 + row * squareSize)
                    # draw
                    surface.blit(label, labelPos)

                # col coordinates
                if row == 7:
                    # color
                    color = theme.bg.dark if (row + col) % 2 == 0 else theme.bg.light
                    # label
                    label = self.config.font.render(ChessSquare.getAlphacol(col), 1, color)
                    labelPos = (col * squareSize + squareSize - 20, screenHeight - 20)
                    # draw
                    surface.blit(label, labelPos)

    def drawPieces(self, surface):
        for row in range(8):
            for col in range(8):
                if self.board.boardSpaces[row][col].hasPiece():
                    piece = self.board.boardSpaces[row][col].piece
                    
                    # Skip drawing the dragged piece
                    if self.pieceDragger.isDragging and piece is self.pieceDragger.piece:
                        continue
                    
                    piece.setTexture(size=80)
                    img = pygame.image.load(piece.texture)
                    imgCenter = col * squareSize + squareSize // 2, row * squareSize + squareSize // 2
                    piece.textureRect = img.get_rect(center=imgCenter)
                    surface.blit(img, piece.textureRect)


    def drawMoves(self, surface):
        theme = self.config.theme

        if self.pieceDragger.isDragging:  
            piece = self.pieceDragger.piece 

            for move in piece.moveList:
                color = theme.moveList.light if (move.end.row + move.end.col) % 2 == 0 else theme.moveList.dark
                rect = (move.end.col * squareSize, move.end.row * squareSize, squareSize, squareSize)
                pygame.draw.rect(surface, color, rect)


    def drawLastMove(self, surface):
        theme = self.config.theme

        if self.board.previousMove:
            start = self.board.previousMove.start
            end = self.board.previousMove.end

            for position in [start, end]:
                color = theme.trace.light if (position.row + position.col) % 2 == 0 else theme.trace.dark
                rect = (position.col * squareSize, position.row * squareSize, squareSize, squareSize)
                pygame.draw.rect(surface, color, rect)

    def play_game():
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        s.connect(('localhost', 5000))
        print(s.recv(1024).decode())

        while True:
            data = s.recv(1024).decode()
            board = chess.Board(data)
            print(board)

            if board.is_game_over():
                break

            if board.turn == chess.WHITE:
                move = input("Enter move: ")
                move = chess.Move.from_uci(move)
                s.sendall(str(move).encode())
            else:
                print("AI is thinking...")
                move = ai_move(board, depth=3)
                s.sendall(str(move).encode())

    print("Game over")
    print("Result:", board.result())

play_game()

    def drawHover(self, surface):
        if self.hoveredSquare:
            color = (180, 180, 180)
            rect = (self.hoveredSquare.col * squareSize, self.hoveredSquare.row * squareSize, squareSize, squareSize)
            pygame.draw.rect(surface, color, rect, width=3)
    
    def switchTurn(self):
        self.currentPlayer = 'white' if self.currentPlayer == 'black' else 'black'

    def setHover(self, row, col):
        self.hoveredSquare = self.board.boardSpaces[row][col]

    def changeTheme(self):
        self.config.changeTheme()

    def playSound(self, captured=False):
        if captured:
            self.config.captureSound.play()
        else:
            self.config.moveSound.play()

    def reset(self):
        self.__init__()