import pygame
import sys
from game import ChessGame
import chess
import random
from graphics import ChessSquare, ChessMove, ChessTheme

screenWidth = 800
screenHeight = 800
boardRows = 8
boardCols = 8
squareSize = screenWidth // boardCols

class Button:
    def __init__(self, text, x, y, width, height, font_size=40, text_color=(255, 255, 255), bg_color=(70, 70, 70)):
        self.text = text
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.font_size = font_size
        self.text_color = text_color
        self.bg_color = bg_color
        self.rect = pygame.Rect(x, y, width, height)

    def draw(self, display):
        pygame.draw.rect(display, self.bg_color, self.rect)
        font = pygame.font.SysFont("Calibri", self.font_size)
        text_render = font.render(self.text, True, self.text_color)
        display.blit(text_render, (self.x + self.width // 2 - text_render.get_width() // 2, self.y + self.height // 2 - text_render.get_height() // 2))

    def is_hover(self, mouse_pos):
        return self.rect.collidepoint(mouse_pos)

class ChessMain:
    def __init__(self):
        pygame.init()
        self.display = pygame.display.set_mode((screenWidth, screenHeight))
        pygame.display.set_caption('Chess')
        self.chessGame = ChessGame()
        self.clock = pygame.time.Clock()
        self.menuBackground = pygame.image.load("menu_background.jpg").convert()
        self.menuBackground = pygame.transform.scale(self.menuBackground, (screenWidth, screenHeight))
        self.online_client = None
        
    def drawMenu(self):
        self.display.blit(self.menuBackground, (0, 0))

    def aboutScreen(self):
            running = True
            font = pygame.font.SysFont("Calibri", 20)
            backButton = Button("Back", 10, screenHeight - 60, 100, 50)

            rules = [
                "Rules of the Game:",
                "1. The game is played on an 8x8 grid.",
                "2. Each player starts with 1 king, 1 queen, 2 rooks, 2 knights, 2 bishops, and 8 pawns.",
                "3. The objective of the game is to checkmate the opponent's king.",
                "4. A player can move a piece to an unoccupied square or capture an opponent's piece.",
                "5. Each type of piece has its own unique movement rules.",
                "",
                "Controls:",
                "Move pieces: Click and drag with the mouse",
                "Change theme: Press 'T'",
                "Restart game: Press 'R'"
            ]

            while running:
                self.display.fill((30, 30, 30))

                for index, line in enumerate(rules):
                    textRender = font.render(line, True, (255, 255, 255))
                    self.display.blit(textRender, (20, 20 + index * 25))

                backButton.draw(self.display)

                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        pygame.quit()
                        sys.exit()

                    mouse_pos = pygame.mouse.get_pos()

                    if event.type == pygame.MOUSEBUTTONDOWN:
                        if backButton.is_hover(mouse_pos):
                            running = False

                pygame.display.update()
                self.clock.tick(60)


    def main_menu(self):
        menu = True
        buttons = [
            Button("Single Player", 100, 200, 200, 50),
            Button("Multiplayer", 100, 300, 200, 50),
            Button("Online", 100, 400, 200, 50),
            Button("About", 10, screenHeight - 60, 100, 50),
            Button("Quit", screenWidth - 160, screenHeight - 60, 150, 50)
        ]
        
        while menu:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()

            self.drawMenu()

            for button in buttons:
                button.draw(self.display)

            mouse_pos = pygame.mouse.get_pos()

            if event.type == pygame.MOUSEBUTTONDOWN:
                for button in buttons:
                    if button.is_hover(mouse_pos):
                        if button.text == "Single Player":
                            pass
                        elif button.text == "Multiplayer":
                            self.gameLoop()
                        elif button.text == "Online":
                            pass
                        elif button.text == "About":
                            self.aboutScreen()
                        elif button.text == "Quit":
                            pygame.quit()
                            sys.exit()

            pygame.display.update()
            self.clock.tick(60)


def minimax(board, depth, alpha, beta, maximizing_player):
    if depth == 0 or board.is_game_over():
        return board.heuristic_value()

    if maximizing_player:
        max_eval = float('-inf')
        for move in board.generate_legal_moves():
            board.push(move)
            eval = minimax(board, depth-1, alpha, beta, False)
            board.pop()
            max_eval = max(max_eval, eval)
            alpha = max(alpha, eval)
            if beta <= alpha:
                break
        return max_eval
    else:
        min_eval = float('inf')
        for move in board.generate_legal_moves():
            board.push(move)
            eval = minimax(board, depth-1, alpha, beta, True)
            board.pop()
            min_eval = min(min_eval, eval)
            beta = min(beta, eval)
            if beta <= alpha:
                break
        return min_eval

def ai_move(board, depth=2):
    best_move = None
    best_eval = float('-inf')
    alpha = float('-inf')
    beta = float('inf')
    for move in board.generate_legal_moves():
        board.push(move)
        eval = minimax(board, depth-1, alpha, beta, False)
        board.pop()
        if eval > best_eval:
            best_eval = eval
            best_move = move
        alpha = max(alpha, eval)
    return best_move

def singleplayer():
    board = chess.Board()
    print(board)

    while not board.is_game_over():
        if board.turn == chess.WHITE:
            move = input("Enter move: ")
            try:
                board.push_san(move)
            except ValueError:
                print("Invalid move, try again")
                continue
        else:
            print("AI is thinking...")
            move = ai_move(board, depth=3)
            board.push(move)
            print("AI plays:", move.uci())
        print(board)

    print("Game over")
    print("Result:", board.result())

singleplayer()


    def gameLoop(self):
        display = self.display
        chessGame = self.chessGame
        chessBoard = self.chessGame.board
        pieceDragger = self.chessGame.pieceDragger

        while True:
            # display methods
            chessGame.drawBackground(display)
            chessGame.drawLastMove(display)
            chessGame.drawMoves(display)
            chessGame.drawPieces(display)
            chessGame.drawHover(display)

            for event in pygame.event.get():

                # mouse click
                if event.type == pygame.MOUSEBUTTONDOWN:
                    pieceDragger.updateMousePosition(event.pos)

                    clickedRow = pieceDragger.mouseY // squareSize
                    clickedCol = pieceDragger.mouseX // squareSize

                    # check if clicked square contains a piece
                    if chessBoard.boardSpaces[clickedRow][clickedCol].hasPiece():
                        piece = chessBoard.boardSpaces[clickedRow][clickedCol].piece
                        # check if clicked piece is valid for current player
                        if piece.color == chessGame.currentPlayer:
                            chessBoard.computeMoves(piece, clickedRow, clickedCol, checkForCheck=True)
                            pieceDragger.saveInitialPosition(event.pos)
                            pieceDragger.startDraggingPiece(piece)

                # mouse movement
                elif event.type == pygame.MOUSEMOTION:
                    motionRow = event.pos[1] // squareSize
                    motionCol = event.pos[0] // squareSize

                    chessGame.setHover(motionRow, motionCol)

                    if pieceDragger.isDragging:
                        pieceDragger.updateMousePosition(event.pos)
                        pieceDragger.updateBlit(display)

                # mouse release
                elif event.type == pygame.MOUSEBUTTONUP:

                    if pieceDragger.isDragging:
                        pieceDragger.updateMousePosition(event.pos)

                        releasedRow = pieceDragger.mouseY // squareSize
                        releasedCol = pieceDragger.mouseX // squareSize

                        # create a potential move
                        initialSquare = ChessSquare(pieceDragger.startRow, pieceDragger.startCol)
                        finalSquare = ChessSquare(releasedRow, releasedCol)
                        move = ChessMove(initialSquare, finalSquare)

                        # check if move is valid
                        if chessBoard.isValidMove(pieceDragger.piece, move):
                            # regular capture
                            capturedPiece = chessBoard.boardSpaces[releasedRow][releasedCol].hasPiece()
                            chessBoard.makeMove(pieceDragger.piece, move)

                            chessBoard.setTrueEnPassant(pieceDragger.piece)

                            # play sound
                            chessGame.playSound(capturedPiece)
                            # switch player
                            chessGame.switchTurn()

                            if self.online_client:
                                self.online_client.send_move_data({'initial': (pieceDragger.startRow, pieceDragger.startCol), 'final': (releasedRow, releasedCol)})

                    pieceDragger.stopDraggingPiece()

                # key press
                elif event.type == pygame.KEYDOWN:
                    # change themes
                    if event.key == pygame.K_t:
                        chessGame.changeTheme()

                    # reset game
                    if event.key == pygame.K_r:
                        chessGame.reset()
                        chessGame = self.chessGame
                        chessBoard = self.chessGame.board
                        pieceDragger = self.chessGame.pieceDragger

                # exit application
                elif event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
            

            pygame.display.update()

chessMain = ChessMain()
chessMain.main_menu()
chessMain.gameLoop()


 
