import os

screenWidth = 800
screenHeight = 800
boardRows = 8
boardCols = 8
squareSize = screenWidth // boardCols

class Piece:
    def __init__(self, pieceName, pieceColor, pieceValue, texture=None, textureRect=None):
        self.name = pieceName
        self.color = pieceColor
        valueSign = 1 if pieceColor == 'white' else -1
        self.value = pieceValue * valueSign
        self.moveList = []
        self.hasMoved = False
        self.texture = texture
        self.setTexture()
        self.textureRect = textureRect

    def setTexture(self, size=80):
        self.texture = os.path.join(
            f'assets/images/imgs-{size}px/{self.color}_{self.name}.png')

    def addMove(self, move):
        self.moveList.append(move)

    def clearMoves(self):
        self.moveList = []

class Pawn(Piece):
    def __init__(self, color):
        self.direction = -1 if color == 'white' else 1
        self.enPassant = False
        super().__init__('pawn', color, 1.0)

class Knight(Piece):
    def __init__(self, color):
        super().__init__('knight', color, 3.0)

class Bishop(Piece):
    def __init__(self, color):
        super().__init__('bishop', color, 3.001)

class Rook(Piece):
    def __init__(self, color):
        super().__init__('rook', color, 5.0)

class Queen(Piece):
    def __init__(self, color):
        super().__init__('queen', color, 9.0)

class King(Piece):
    def __init__(self, color):
        self.leftCastleRook = None
        self.rightCastleRook = None
        super().__init__('king', color, 10000.0)